module definitions
    type tracking_cloud
      integer np,ntracks
      real, dimension(:), pointer :: time
      real, dimension(:,:),pointer :: x,y,z,press,temp,velo,veloZ
      integer,dimension(:),pointer :: step
      integer,dimension(:,:),pointer :: tracks
    end type tracking_cloud

end module definitions


!==============================================================================!
!==============================================================================!
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!==============================================================================!
!==============================================================================!
!                                                                              |
!              TRACKING CLOUD POST PROCESSING   Jan. 2015                      |
!                                                                              |
!==============================================================================!
!==============================================================================!
!                                                                              |
! This routine reads in the tracking output files, extracts the surfaced       |
! tracks, does variable calculation and writes ASCII vtk files.                |
!                                                                              |
! CAUTION: Particle ID (Douar) and Point Number (VTK) differ: DOUAR numbering  |
!     starts at 1, VTK numbering starts at 0.                                  |

program post_tracking

use definitions

implicit none

type (tracking_cloud) :: trcl
 
!input parameters from input_tracking_output.txt
character(150) :: input_dir
integer :: output_mode
integer :: output_scaled
integer :: output_first
integer :: output_last
integer :: output_step
integer :: keep_nstep
integer :: min_track_length
integer :: input_first
integer :: input_step
integer :: output_velo
integer :: output_veloz
integer :: output_temp
integer :: output_press
integer :: output_time
integer :: output_istep
integer :: output_ID

!buffer variables for reading
real :: read_time,lref,veloref,rhoref,tempscale
integer :: read_step,read_ntracks
integer,dimension(:,:),allocatable :: read_tracks
integer,dimension(:),allocatable :: keep_tracks

!other variables
integer :: i,k,l,err,dummyint,itrack,ipoint
integer :: output_nstep,input_nstep,input_last,output_any
integer :: write_nstep,write_points,write_tracks
logical :: keepme

character(4) :: cistep

write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,'(a)') '!                                                                              |'
write (*,'(a)') '!          8888888b.   .d88888b.  888     888       d8888 8888888b.            |'
write (*,'(a)') '!          888  "Y88b d88P" "Y88b 888     888      d88888 888   Y88b           |'
write (*,'(a)') '!          888    888 888     888 888     888     d88P888 888    888           |'
write (*,'(a)') '!          888    888 888     888 888     888    d88P 888 888   d88P           |'
write (*,'(a)') '!          888    888 888     888 888     888   d88P  888 8888888P"            |'
write (*,'(a)') '!          888    888 888     888 888     888  d88P   888 888 T88b             |'
write (*,'(a)') '!          888  .d88P Y88b. .d88P Y88b. .d88P d8888888888 888  T88b            |'
write (*,'(a)') '!          8888888P"   "Y88888P"   "Y88888P" d88P     888 888   T88b           |'
write (*,'(a)') '!                                                                              |'
write (*,'(a)') '!             DOUAR-WSMP post-processor for the tracking cloud                 |'
write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,*) ''

!------------------------------------------------------------------------------|
!    Input parameters
!------------------------------------------------------------------------------|

!Reading input parameters from post-input-file:

open(unit=10,file='input_tracking_outputs.txt',status='old')

read (10,*) input_dir
read (10,*) output_mode
read (10,*) output_scaled
read (10,*) output_first
read (10,*) output_last
read (10,*) output_step
read (10,*) keep_nstep
read (10,*) min_track_length
!keep_nstep has to be less than output_step
keep_nstep = min(keep_nstep,output_step-1)
read (10,*) input_first
read (10,*) input_step
read (10,*)  !here come the age parameters
read (10,*)  !here come the track parameters
read (10,*) output_velo
read (10,*) output_veloz
read (10,*) output_temp
read (10,*) output_press
read (10,*) output_time
read (10,*) output_istep
read (10,*) output_ID

close (10)

!calculating and andjusting input and output timesteps
output_nstep = (output_last-output_first)/output_step+1
output_last = output_first+(output_nstep-1)*output_step

input_last = output_last
input_nstep = (input_last-input_first)/input_step+1
input_last = input_first+(input_nstep-1)*input_step

!writign output parameters to screen
write (*,'(a,a)') 'Input Directory: ',input_dir
write (*,'(a,i2,a,i3,a,i3,a,i3)') 'Input Timesteps: Reading ',input_nstep,' input files & 
from step ',input_first,' to ',input_last,' in steps of ',input_step
write (*,'(a,i2,a,i3,a,i3,a,i3)') 'Output Timesteps: Generating ',output_nstep,' output files & 
from step ',output_first,' to ',output_last,' in steps of ',output_step
write (*,'(a,i2,a)') 'accumulating ',keep_nstep,' steps for surfaced tracks'
write (*,'(a,i1)') 'Output mode: ',output_mode
write (*,'(a,l)') 'Write natural units:     ',(output_scaled==1)
write (*,'(a,l)') 'Write track velocity:    ',(output_velo==1)
write (*,'(a,l)') 'Write track z velocity:  ',(output_veloz==1)
write (*,'(a,l)') 'Write track temperature: ',(output_temp==1)
write (*,'(a,l)') 'Write track pressure:    ',(output_press==1)
write (*,'(a,l)') 'Write track time:        ',(output_time==1)
write (*,'(a,l)') 'Write track step number: ',(output_istep==1)
write (*,'(a,l)') 'Write particle/track ID: ',(output_ID==1)
write (*,'(a)') ''
write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,'(a)') ''

!------------------------------------------------------------------------------|
!    Reading Tracking Cloud Files
!------------------------------------------------------------------------------|

!reading input file header for array sizes
call int_to_char (cistep,4,input_last)

open (unit=20,file=trim(input_dir)//'tracking_t'//cistep//'.bin',status='old',form='unformatted')
read (20) trcl%np,read_ntracks,read_step,read_time
read (20) lref,veloref,rhoref,tempscale
close (20)

!allocating master arrays for input
allocate (trcl%x(trcl%np,input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%x'
allocate (trcl%y(trcl%np,input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%y'
allocate (trcl%z(trcl%np,input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%z'
allocate (trcl%temp(trcl%np,input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%temp'
allocate (trcl%press(trcl%np,input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%press'
allocate (trcl%step(input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%step'
allocate (trcl%time(input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%time'
allocate (read_tracks(read_ntracks,3),stat=err); if (err.ne.0) stop 'Error alloc read_tracks'

!read all input files and save in collective arrays
k = 1
do i = input_first,input_last,input_step
    !read all information for this time step
    call int_to_char (cistep,4,i)
    open (unit=20,file=trim(input_dir)//'tracking_t'//cistep//'.bin',status='old',form='unformatted')
    
    read (20) dummyint,dummyint,trcl%step(k),trcl%time(k)
    read (20) 
    read (20) trcl%x(:,k)
    read (20) trcl%y(:,k)
    read (20) trcl%z(:,k)
    read (20) trcl%temp(:,k)
    read (20) trcl%press(:,k)
    
    !read track information only from last timestep
    if (i==input_last) then
        write (*,'(a)') 'All surfaced tracks:'
        write (*,'(a)') 'track ID, Particle ID, start, end'
        do l=1,read_ntracks 
            read (20) read_tracks(l,:)
            write (*,'(4i6)') l, read_tracks(l,:)
        end do
        write (*,'(a)') ''
    end if

    !advance column in master array
    k = k+1

    close (20)
end do

write (*,'(a)') 'Finished reading input files'
write (*,'(a)') ''

!select only tracks that surface at an output timestep or up to keep_nstep
!timesteps before each output timestep and are at least min_track_lenght 
!timesteps long.
!Then adjust track timesteps to actual position in master array.
!The offset k has to be recorded in order to associate the track with the
!correct timestep.

allocate (keep_tracks(read_ntracks),stat=err); if (err.ne.0) stop 'Error alloc keep_tracks'
keep_tracks = 0
trcl%ntracks = 0
do k=0,keep_nstep
    do i=1,read_ntracks
        keepme = read_tracks(i,3) >= output_first-k
        keepme = keepme .AND. (mod((read_tracks(i,3)+k-output_first),output_step)==0)
        keepme = keepme .AND. read_tracks(i,3)-read_tracks(i,2)+1 >= min_track_length
        if (keepme) then
            keep_tracks(i) = 1000+k
            trcl%ntracks = trcl%ntracks+1
        end if
    end do
end do

!allocate tracks array of needed length

if (trcl%ntracks > 0) then
    allocate (trcl%tracks(trcl%ntracks,6),stat=err); if (err.ne.0) stop 'Error alloc trcl%tracks'
    trcl%tracks = 0

    !copy relevant tracks into new track array
    !adjust track timsteps to position in new track array
    !always round up for start and round down for end in order not to include parts of other tracks
    itrack=1
    write (*,'(a)') 'Selected surfaced tracks:'
    write (*,'(a)') '*: in master array'
    write (*,'(a)') 'Particle ID, start*, end*, length*, output timestep, track ID'
    do i=1,read_ntracks
        if (keep_tracks(i)>0) then
            !1st column is particle ID
            trcl%tracks(itrack,1) = read_tracks(i,1)
            !2nd column is start in master array, round up
            trcl%tracks(itrack,2)= max(1,ceiling(1.0+(read_tracks(i,2)-input_first)/(1.0*input_step)))
            !3rd columd is end in master array, round down
            trcl%tracks(itrack,3)= max(1,1+(read_tracks(i,3)-input_first)/input_step) !integer division
            !4th column is track length (number of points in master array)
            trcl%tracks(itrack,4) = trcl%tracks(itrack,3)-trcl%tracks(itrack,2)+1
            !5th column is Douar timestep of surfacing, upshifted for those tracks 
            !within keep_nstep
            trcl%tracks(itrack,5) = read_tracks(i,3)+(keep_tracks(i)-1000)
            !6th column is track ID, i.e. the row of the original track array from input file
            trcl%tracks(itrack,6) = i

            write (*,'(6i6)') trcl%tracks(itrack,:)
            itrack = itrack+1
        end if
    end do
else
    write (*,'(a)') 'No surfaced tracks'
end if

deallocate (read_tracks)
deallocate (keep_tracks)

!------------------------------------------------------------------------------|
!    Scaling Values to natural units
!------------------------------------------------------------------------------|
!check what age prediction needs...
!units of scaling parameters:
!lref: km
!veloref: mm/yr = km/Myr
!rhoref: kg/m^3
!tempscale: °C

if (output_scaled==1) then
    do k=1,input_nstep
        do i=1,trcl%np
            trcl%x(i,k) = trcl%x(i,k)*lref !pos in km
            trcl%y(i,k) = trcl%y(i,k)*lref
            trcl%z(i,k) = trcl%z(i,k)*lref
            trcl%temp(i,k) = trcl%temp(i,k)*tempscale !temp in C
            trcl%press(i,k) = trcl%press(i,k)*1.0e-3*9.81*rhoref*lref !pressure in MPa
        end do
        trcl%time(k) = trcl%time(k)*lref/veloref !time in Myr
    end do
end if


!------------------------------------------------------------------------------|
!    Calculating Output values
!------------------------------------------------------------------------------|

!track values
velo: if (output_velo==1) then
    allocate (trcl%velo(trcl%np,input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%velo$'
    do i=1,trcl%np
            trcl%velo(i,1) = sqrt( & 
                (trcl%x(i,2)-trcl%x(i,1))**2+(trcl%y(i,k)-trcl%y(i,1))**2+(trcl%z(i,k)-trcl%z(i,1))**2) &
                /(trcl%time(2)-trcl%time(1))
    end do
    do i=1,trcl%np
        do k=2,input_nstep
            trcl%velo(i,k) = sqrt( & 
                (trcl%x(i,k)-trcl%x(i,k-1))**2+(trcl%y(i,k)-trcl%y(i,k-1))**2+(trcl%z(i,k)-trcl%z(i,k-1))**2) &
                /(trcl%time(k)-trcl%time(k-1))
        end do
    end do
end if velo


veloz: if (output_veloz==1) then
    allocate (trcl%veloz(trcl%np,input_nstep),stat=err); if (err.ne.0) stop 'Error alloc trcl%veloz$'
    do i=1,trcl%np
        trcl%veloz(i,1) = (trcl%z(i,2)-trcl%z(i,1))/(trcl%time(2)-trcl%time(1))
    end do
    do i=1,trcl%np
        do k=2,input_nstep
            trcl%veloz(i,k) = (trcl%z(i,k)-trcl%z(i,k-1))/(trcl%time(k)-trcl%time(k-1))
        end do
    end do
end if veloz

!------------------------------------------------------------------------------|
!    Checking for NaNs and variable ranges
!------------------------------------------------------------------------------|


do k=1,input_nstep
    if (isnan(trcl%time(k))) then
        trcl%time(k)=1.0e38
    else
        if (     trcl%time(k)> 1.0e37) trcl%time(k)=1.0e37
        if (     trcl%time(k)<-1.0e37) trcl%time(k)=-1.0e37
        if (abs(trcl%time(k))<1.0e-37) trcl%time(k)=0.0
    end if
    do i=1,trcl%np
        if (isnan(trcl%temp(i,k))) then
            trcl%temp(i,k)=1.0e38
        else
            if (     trcl%temp(i,k)> 1.0e37) trcl%temp(i,k)=1.0e37
            if (     trcl%temp(i,k)<-1.0e37) trcl%temp(i,k)=-1.0e37
            if (abs(trcl%temp(i,k))<1.0e-37) trcl%temp(i,k)=0.0
        end if
        if (isnan(trcl%press(i,k))) then
            trcl%press(i,k)=1.0e38
        else
            if (     trcl%press(i,k)> 1.0e37) trcl%press(i,k)=1.0e37
            if (     trcl%press(i,k)<-1.0e37) trcl%press(i,k)=-1.0e37
            if (abs(trcl%press(i,k))<1.0e-37) trcl%press(i,k)=0.0
        end if
        if (output_velo==1) then
            if (isnan(trcl%velo(i,k))) then
                trcl%velo(i,k)=1.0e38
            else
                if (     trcl%velo(i,k)> 1.0e37) trcl%velo(i,k)=1.0e37
                if (     trcl%velo(i,k)<-1.0e37) trcl%velo(i,k)=-1.0e37
                if (abs(trcl%velo(i,k))<1.0e-37) trcl%velo(i,k)=0.0
            end if
        end if
        if (output_veloz==1) then
            if (isnan(trcl%veloz(i,k))) then
                trcl%veloz(i,k)=1.0e38
            else
                if (     trcl%veloz(i,k)> 1.0e37) trcl%veloz(i,k)=1.0e37
                if (     trcl%veloz(i,k)<-1.0e37) trcl%veloz(i,k)=-1.0e37
                if (abs(trcl%veloz(i,k))<1.0e-37) trcl%veloz(i,k)=0.0
            end if
        end if
    end do
end do

!------------------------------------------------------------------------------|
!    Writing Output Files
!------------------------------------------------------------------------------|
output_any = output_velo+output_veloz+output_temp+output_press+output_time+output_istep+output_ID

if (output_mode==2 .and. trcl%ntracks>0) then
!output ages and surfaced tracks for each output timestep
    do l=output_first,output_last,output_step
        !count number of tracks and number of points in those for this timestep
        write_tracks = 0
        write_points = 0
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write_tracks = write_tracks+1
                write_points = write_points+trcl%tracks(i,4)
            end if
        end do

        if (write_tracks>0) then
            call int_to_char (cistep,4,l)
            open (unit=30,file=trim(input_dir)//'tracks_t'//cistep//'.vtk',status='replace')

            !write header
            write(30,'(a)') '# vtk DataFile Version 3.1'
            write(30, '(a)') 'DOUAR tracking cloud output of surfaced tracks'
            write(30, '(a)') 'ASCII'
            write(30, '(a)') 'DATASET UNSTRUCTURED_GRID'
            write(30, '(a)') ''

            !write point coordinates
            write(30, '(a,i,a)') 'POINTS ', write_points, ' FLOAT'
            do i=1,trcl%ntracks
            !go through all tracks
                if (trcl%tracks(i,5)==l) then
                !if track surfaces at this timestep:
                    do k=trcl%tracks(i,2),trcl%tracks(i,3)
                    !write all point positions within track extent
                        write(30,'(3ES12.5E2)') trcl%x(trcl%tracks(i,1),k), &
                            trcl%y(trcl%tracks(i,1),k),trcl%z(trcl%tracks(i,1),k)
                    end do
                end if
            end do
            write(30, '(a)') ''

            !write cell data
            write(30, '(a,i10,i10)') 'CELLS ', write_tracks, write_points+write_tracks
            ipoint = 0
            do i=1,trcl%ntracks
                if (trcl%tracks(i,5)==l) then
                    !write length of track
                    write(30, '(i)',advance='no') trcl%tracks(i,4)
                    do k=trcl%tracks(i,2),trcl%tracks(i,3)
                        !write point ID
                        write(30, '(i)', advance='no') ipoint
                        !advance point ID
                        ipoint = ipoint+1
                    end do
                    !end of track
                    write (30, '(a)') '' 
                end if
            end do
            write (30, '(a)') '' 
            

            !write cell data type, 4=polyline
            write(30, '(a,i10)') 'CELL_TYPES ',write_tracks
            do i = 1,write_tracks
                write(30, '(i2)') 4
            end do
            write (30, '(a)') '' 

            !write point datasets
            if (output_any > 0) write (30,'(a,i)') 'POINT_DATA ',write_points
            
            if (output_velo==1) then
                write (30,'(a)') 'SCALARS Velo FLOAT 1'
                write (30,'(a)') 'LOOKUP_TABLE default'
                do i=1,trcl%ntracks
                    if (trcl%tracks(i,5)==l) then
                        do k=trcl%tracks(i,2),trcl%tracks(i,3)
                            write(30,'(3ES12.5E2)') trcl%velo(trcl%tracks(i,1),k)
                        end do
                    end if
                end do
                write(30, '(a)') ''
            end if

            if (output_veloz==1) then
                write (30,'(a)') 'SCALARS VeloZ FLOAT 1'
                write (30,'(a)') 'LOOKUP_TABLE default'
                do i=1,trcl%ntracks
                    if (trcl%tracks(i,5)==l) then
                        do k=trcl%tracks(i,2),trcl%tracks(i,3)
                            write(30,'(3ES12.5E2)') trcl%veloz(trcl%tracks(i,1),k)
                        end do
                    end if
                end do
                write(30, '(a)') ''
            end if

            if (output_temp==1) then
                write (30,'(a)') 'SCALARS Temperature FLOAT 1'
                write (30,'(a)') 'LOOKUP_TABLE default'
                do i=1,trcl%ntracks
                    if (trcl%tracks(i,5)==l) then
                        do k=trcl%tracks(i,2),trcl%tracks(i,3)
                            write(30,'(3ES12.5E2)') trcl%temp(trcl%tracks(i,1),k)
                        end do
                    end if
                end do
                write(30, '(a)') ''
            end if

            if (output_press==1) then
                write (30,'(a)') 'SCALARS Pressure FLOAT 1'
                write (30,'(a)') 'LOOKUP_TABLE default'
                do i=1,trcl%ntracks
                    if (trcl%tracks(i,5)==l) then
                        do k=trcl%tracks(i,2),trcl%tracks(i,3)
                            write(30,'(3ES12.5E2)') trcl%press(trcl%tracks(i,1),k)
                        end do
                    end if
                end do
                write(30, '(a)') ''
            end if

            if (output_time==1) then
                write (30,'(a)') 'SCALARS Time FLOAT 1'
                write (30,'(a)') 'LOOKUP_TABLE default'
                do i=1,trcl%ntracks
                    if (trcl%tracks(i,5)==l) then
                        do k=trcl%tracks(i,2),trcl%tracks(i,3)
                            write(30,'(3ES12.5E2)') trcl%time(k)
                        end do
                    end if
                end do
                write(30, '(a)') ''
            end if

            if (output_istep==1) then
                write (30,'(a)') 'SCALARS Step INTEGER 1'
                write (30,'(a)') 'LOOKUP_TABLE default'
                do i=1,trcl%ntracks
                    if (trcl%tracks(i,5)==l) then
                        do k=trcl%tracks(i,2),trcl%tracks(i,3)
                            write(30,'(i)') trcl%step(k)
                        end do
                    end if
                end do
                write(30, '(a)') ''
            end if

            if (output_ID==1) then
                write (30,'(a)') 'SCALARS Track_ID INTEGER 1'
                write (30,'(a)') 'LOOKUP_TABLE default'
                do i=1,trcl%ntracks
                    if (trcl%tracks(i,5)==l) then
                        do k=trcl%tracks(i,2),trcl%tracks(i,3)
                            write(30,'(i)') trcl%tracks(i,6)
                        end do
                    end if
                end do
                write(30, '(a)') ''
            end if

            close (30)
        else
            write (*,'(a,i4)') 'No surfaced tracks for step ', l
        end if
    end do
end if

if (output_mode==3) then
!output all tracks
    do l=output_first,output_last,output_step
        call int_to_char (cistep,4,l)
        open (unit=30,file=trim(input_dir)//'alltracks_t'//cistep//'.vtk',status='replace')

        !calculate part of array to write
        write_nstep = (l-input_first)/input_step+1

        !write header
        write(30,'(a)') '# vtk DataFile Version 3.1'
        write(30, '(a)') 'DOUAR tracking cloud output of all tracks'
        write(30, '(a)') 'ASCII'
        write(30, '(a)') 'DATASET UNSTRUCTURED_GRID'
        write(30, '(a)') ''

        !write point coordinates
        write(30, '(a,i,a)') 'POINTS ', trcl%np*write_nstep, ' FLOAT'
        do i=1,trcl%np
            do k=1,write_nstep
                write(30,'(3ES12.5E2)') trcl%x(i,k),trcl%y(i,k),trcl%z(i,k)
            end do
        end do
        write(30, '(a)') ''

        !write cell data
        write(30, '(a,i10,i10)') 'CELLS ',trcl%np, trcl%np*(write_nstep+1)
        do i = 0,trcl%np-1
            write(30, '(i)',advance='no') write_nstep
            do k = 1,write_nstep
                write(30, '(i)', advance='no') i*write_nstep+k-1
            end do
            write (30, '(a)') '' 
        end do
        write (30, '(a)') '' 

        !write cell data type, 4=polyline
        write(30, '(a,i10)') 'CELL_TYPES ',trcl%np
        do i = 1,trcl%np
            write(30, '(i2)') 2
        end do
        write (30, '(a)') '' 

        !write point datasets
        if (output_any > 0) write (30,'(a,i)') 'POINT_DATA ',trcl%np*write_nstep
        
        if (output_velo==1) then
            write (30,'(a)') 'SCALARS Velo FLOAT 1'
            write (30,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%np
                do k=1,write_nstep
                    write(30,'(ES)') trcl%velo(i,k)
                end do
            end do
            write(30, '(a)') ''
        end if

        if (output_veloz==1) then
            write (30,'(a)') 'SCALARS VeloZ FLOAT 1'
            write (30,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%np
                do k=1,write_nstep
                    write(30,'(ES12.5E2)') trcl%veloz(i,k)
                end do
            end do
            write(30, '(a)') ''
        end if

        if (output_temp==1) then
            write (30,'(a)') 'SCALARS Temperature FLOAT 1'
            write (30,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%np
                do k=1,write_nstep
                    write(30,'(ES12.5E2)') trcl%temp(i,k)
                end do
            end do
            write(30, '(a)') ''
        end if

        if (output_press==1) then
            write (30,'(a)') 'SCALARS Pressure FLOAT 1'
            write (30,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%np
                do k=1,write_nstep
                    write(30,'(ES12.5E2)') trcl%press(i,k)
                end do
            end do
            write(30, '(a)') ''
        end if

        if (output_time==1) then
            write (30,'(a)') 'SCALARS Time FLOAT 1'
            write (30,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%np
                do k=1,write_nstep
                    write(30,'(ES12.5E2)') trcl%time(k)
                end do
            end do
            write(30, '(a)') ''
        end if

        if (output_istep==1) then
            write (30,'(a)') 'SCALARS Step INTEGER 1'
            write (30,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%np
                do k=1,write_nstep
                    write(30,'(i)') trcl%step(k)
                end do
            end do
            write(30, '(a)') ''
        end if

        if (output_ID==1) then
            write (30,'(a)') 'SCALARS Particle_ID INTEGER 1'
            write (30,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%np
                do k=1,write_nstep
                    write(30,'(i)') i
                end do
            end do
            write(30, '(a)') ''
        end if

        close (30)
    end do
end if





deallocate (trcl%x)   
deallocate (trcl%y)
deallocate (trcl%z)
deallocate (trcl%temp)
deallocate (trcl%press)
deallocate (trcl%step)
deallocate (trcl%time)
if (trcl%ntracks>0) deallocate (trcl%tracks)
if (output_velo==1) deallocate (trcl%velo)
if (output_veloz==1) deallocate (trcl%veloz)


end
